@@ -25,7 +25,7 @@ import (
 	"kubevirt.io/containerized-data-importer/tests/framework"
 	"kubevirt.io/containerized-data-importer/tests/utils"
 
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
+	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 
 	ocpconfigv1 "github.com/openshift/api/config/v1"
 )
@@ -285,13 +285,10 @@ var _ = Describe("Import Proxy tests", func() {
 
 			By("Creating new datavolume")
 			dv := utils.NewDataVolumeWithRegistryImport("import-dv", "1Gi", fmt.Sprintf(utils.TinyCoreIsoRegistryURL, f.CdiInstallNs))
-			if len(dv.GetAnnotations()) == 0 {
-				dv.SetAnnotations(make(map[string]string, 0))
-			}
 			dv.Annotations[controller.AnnPodRetainAfterCompletion] = "true"
 			cm, err := utils.CopyRegistryCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).To(BeNil())
-			dv.Spec.Source.Registry.CertConfigMap = cm
+			dv.Spec.Source.Registry.CertConfigMap = &cm
 			dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
 			Expect(err).To(BeNil())
 			dvName = dv.Name
